@@ -72,6 +72,8 @@ static gboolean get_supported_messages_flag;
 static gboolean reset_flag;
 static gboolean noop_flag;
 static gchar *bind_mux_str;
+static gchar *set_ip_family_str;
+static gboolean get_channel_rates_flag;
 
 static GOptionEntry entries[] = {
     { "wds-start-network", 0, 0, G_OPTION_ARG_STRING, &start_network_str,
@@ -146,6 +148,14 @@ static GOptionEntry entries[] = {
       "Bind qmux data port to controller device (allowed keys: mux-id, ep-iface-number) to be used with `--client-no-release-cid'",
       "[\"key=value,...\"]"
     },
+    { "wds-set-ip-family", 0, 0, G_OPTION_ARG_STRING, &set_ip_family_str,
+      "Set IP family",
+      "[4|6]"
+    },
+    { "wds-get-channel-rates", 0, 0, G_OPTION_ARG_NONE, &get_channel_rates_flag,
+      "Get channel data rates",
+      NULL
+    },
     { "wds-noop", 0, 0, G_OPTION_ARG_NONE, &noop_flag,
       "Just allocate or release a WDS client. Use with `--client-no-release-cid' and/or `--client-cid'",
       NULL
@@ -180,6 +190,7 @@ qmicli_wds_options_enabled (void)
     n_actions = (!!start_network_str +
                  !!stop_network_str +
                  !!bind_mux_str +
+                 !!set_ip_family_str +
                  get_current_settings_flag +
                  get_packet_service_status_flag +
                  get_packet_statistics_flag +
@@ -194,6 +205,7 @@ qmicli_wds_options_enabled (void)
                  !!set_autoconnect_settings_str +
                  get_supported_messages_flag +
                  reset_flag +
+                 !!get_channel_rates_flag +
                  noop_flag);
 
     if (n_actions > 1) {
@@ -1687,6 +1699,79 @@ bind_mux_data_port_ready (QmiClientWds *client,
     operation_shutdown (TRUE);
 }
 
+static void
+set_ip_family_ready (QmiClientWds *client,
+                     GAsyncResult *res)
+{
+    QmiMessageWdsSetIpFamilyOutput *output;
+    GError *error = NULL;
+
+    output = qmi_client_wds_set_ip_family_finish (client, res, &error);
+    if (!output) {
+        g_printerr ("error: operation failed: %s\n", error->message);
+        g_error_free (error);
+        operation_shutdown (FALSE);
+        return;
+    }
+
+    if (!qmi_message_wds_set_ip_family_output_get_result (output, &error)) {
+        g_printerr ("error: couldn't set IP family: %s\n", error->message);
+        g_error_free (error);
+        qmi_message_wds_set_ip_family_output_unref (output);
+        operation_shutdown (FALSE);
+        return;
+    }
+
+    qmi_message_wds_set_ip_family_output_unref (output);
+    operation_shutdown (TRUE);
+}
+
+static void
+get_channel_rates_ready (QmiClientWds *client,
+                         GAsyncResult *res)
+{
+    QmiMessageWdsGetChannelRatesOutput *output;
+    guint32 txrate = 0, rxrate = 0, maxtxrate = 0, maxrxrate = 0;
+    GError *error = NULL;
+
+    output = qmi_client_wds_get_channel_rates_finish (client, res, &error);
+    if (!output) {
+        g_printerr ("error: operation failed: %s\n", error->message);
+        g_error_free (error);
+        operation_shutdown (FALSE);
+        return;
+    }
+
+    if (!qmi_message_wds_get_channel_rates_output_get_result (output, &error)) {
+        g_printerr ("error: couldn't get channel rates: %s\n",
+                    error->message);
+        g_error_free (error);
+        qmi_message_wds_get_channel_rates_output_unref (output);
+        operation_shutdown (FALSE);
+        return;
+    }
+
+    g_print ("Channel data rates:\n");
+
+    qmi_message_wds_get_channel_rates_output_get_channel_rates (output,
+                                                                &txrate,
+                                                                &rxrate,
+                                                                &maxtxrate,
+                                                                &maxrxrate,
+                                                                NULL);
+    g_print ("\tCurrent TX rate: %ubps\n"
+             "\tCurrent RX rate: %ubps\n"
+             "\tMax TX rate:     %ubps\n"
+             "\tMax RX rate:     %ubps\n",
+             txrate,
+             rxrate,
+             maxtxrate,
+             maxrxrate);
+
+    qmi_message_wds_get_channel_rates_output_unref (output);
+    operation_shutdown (TRUE);
+}
+
 void
 qmicli_wds_run (QmiDevice *device,
                 QmiClientWds *client,
@@ -1762,6 +1847,35 @@ qmicli_wds_run (QmiDevice *device,
         return;
     }
 
+    /* Request to set IP family? */
+    if (set_ip_family_str) {
+        QmiMessageWdsSetIpFamilyInput *input;
+
+        input = qmi_message_wds_set_ip_family_input_new ();
+        switch (atoi (set_ip_family_str)) {
+        case 4:
+            qmi_message_wds_set_ip_family_input_set_preference (input, QMI_WDS_IP_FAMILY_IPV4, NULL);
+            break;
+        case 6:
+            qmi_message_wds_set_ip_family_input_set_preference (input, QMI_WDS_IP_FAMILY_IPV6, NULL);
+            break;
+        default:
+            g_printerr ("error: unknown IP type '%s' (not 4 or 6)\n",
+                        set_ip_family_str);
+            operation_shutdown (FALSE);
+            return;
+        }
+        g_debug ("Asynchronously set IP family...");
+        qmi_client_wds_set_ip_family (client,
+                                      input,
+                                      10,
+                                      ctx->cancellable,
+                                      (GAsyncReadyCallback) set_ip_family_ready,
+                                      NULL);
+        qmi_message_wds_set_ip_family_input_unref (input);
+        return;
+    }
+
     /* Request to get current settings? */
     if (get_current_settings_flag) {
         QmiMessageWdsGetCurrentSettingsInput *input;
@@ -2002,6 +2116,18 @@ qmicli_wds_run (QmiDevice *device,
         return;
     }
 
+    /* Request to get channel data rates? */
+    if (get_channel_rates_flag) {
+        g_debug ("Asynchronously getting channel data rates...");
+        qmi_client_wds_get_channel_rates (client,
+                                          NULL,
+                                          10,
+                                          ctx->cancellable,
+                                          (GAsyncReadyCallback)get_channel_rates_ready,
+                                          NULL);
+        return;
+    }
+
     /* Just client allocate/release? */
     if (noop_flag) {
         g_idle_add (noop_cb, NULL);
